/*
   MODULE NAME        ctgclient.h

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI Version 2
                      Application Programming Interface
                      SupportPac CA0B

   Licensed Materials - Property of IBM
   
   Supportpac CA0B

   (c) Copyright IBM Corp. 2011, 2012 All Rights Reserved.
   
   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with
   IBM Corp.
   
   NOTES :-

   To ensure the correct sections of this header are included, the
   following constant must be defined at compile time:

     CICS_AIX  -  if building an AIX application
*/
/**
* @file
* Defines functions and typedefs for connecting to a Gateway daemon
* and controlling API tracing.
*/

#ifndef _CTGCLIENT_H
#define _CTGCLIENT_H

/* Include stddef.h for size_t type definition */
#include <stddef.h>

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif
/*********************************************************************/
/*             CICS TG ECI Version 2 General constants               */
/*********************************************************************/
/** CICS TG ECI V2 the API version identifier. This is the version
 *  of the API and should not be confused with the ECI Version that
 *  is included in the ECI parameter block field eci_version.
 */
#define CTG_API_VERSION           "2.0.1.0"

/** CICS TG ECI V2 the DLL version identifier. This is the build version
 *  of the DLL and should not be confused with the API Version.
 */
#define CTG_DLL_VERSION           "1.0.0.0"

/** The maximum length of a string returned by
 *  CTG_getRcString
 */
#define CTG_MAX_RCSTRING           40

/** Null Gateway daemon token  */
#define CTG_NULL_GWTOK             NULL



/*********************************************************************/
/*           CICS TG ECI Version 2 Connection functions              */
/*********************************************************************/
/**
 * CTG_ConnToken_t represents a connection to a specific
 * Gateway daemon.
 */
typedef struct _CTG_ConnToken_t * CTG_ConnToken_t;

/**
 * Establish a remote ECI version 2 connection to a Gateway daemon. This 
 * function takes a character pointer for the hostname, an integer for 
 * the target port number and a pointer to a Gateway token and an 
 * integer for the connection timeout. It establishes a connection to a 
 * Gateway daemon Client protocol handler using the specified hostname 
 * and port number. If the return code is successful the Gateway token 
 * represents the connection to the specified Gateway daemon, this 
 * Gateway token is required as an input parameter for the other API 
 * calls. 
 *
 * This API supports CICS TG remote mode operation. The CICS TG "local" 
 * protocol for local mode operation is not supported. 
 *
 * @param[in] address The character string containing the hostname or 
 *       address of the target Gateway daemon.
 * @param[in] port An int for the port of the target Gateway daemon. 
 * @param[out] gwTokPtr Pointer to the new Gateway token.
 * @param[in] connTimeout The connection timeout in seconds, enter zero 
 *       for no timeout.
 * @returns An integer value identifies whether the call was successful 
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADPORT
 * \li #CTG_ERR_CONNECTFAILED
 * \li #CTG_ERR_BADGWTOKLIST
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_BADHOST
 * \li #CTG_ERR_NULLADDRESS
 */
int CICSCALL CTG_openRemoteGatewayConnection(char * address,
                                             int port,
                                             CTG_ConnToken_t * gwTokPtr,
                                             int connTimeout);

/**
 * Closes the connection to the Gateway daemon associated with the
 * specified Gateway token.  After successful completion, resources
 * associated with the Gateway daemon connection are released.
 *
 * When the Gateway token is closed, synchronous requests that are in
 * progress can complete and replies for outstanding asynchronous
 * requests can be retrieved using any of the get reply calls. No new
 * ECI_SYNC or ECI_ASYNC calls can be issued after the close process
 * has started.
 *
 * When a Gateway token is closing, and all outstanding replies have
 * been retrieved, any outstanding ECI_GET_REPLY_WAIT calls end with
 * CTG_ERR_GWTOK_CLOSED.
 *
 * This function waits until all synchronous requests are complete
 * and all replies have been retrieved.
 *
 * @param[in] gwTokPtr The reference to the open Gateway connection
 * @returns An integer value identifies whether the call was successful 
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLGWTOK
 * \li #CTG_ERR_BADGWTOK
 * \li #CTG_ERR_PIDMISMATCH
 * \li #CTG_ERR_NULLGWTOKPTR
 */
int CICSCALL CTG_closeGatewayConnection(CTG_ConnToken_t * gwTokPtr);

/**
 * Attempts to free all resources held by the API, including open 
 * Gateway connections. This function can be called in the event of a
 * severe application error, enabling a controlled shutdown even if all
 * Gateway tokens have been lost.
 *
 * Any outstanding ECI_GET_REPLY_WAIT and ECI_GET_SPECIFIC_REPLY_WAIT
 * calls end with CTG_ERR_GWTOK_CLOSED. No new calls can be issued on
 * any Gateway token after the close process has started.
 *
 * @returns An integer value identifies whether the call was successful 
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADGWTOKLIST
 */
int CICSCALL CTG_closeAllGatewayConnections();


/*********************************************************************/
/*             CICS TG ECI Version 2 Utility functions               */
/*********************************************************************/

/**
 * Get the current trace level. Takes a pointer to an int variable, and
 * sets that variable to the current API trace level.
 *
 * @param[out] traceStatePtr output value, a pointer to an int
 * variable that is set with API trace status.
 * \n Valid values are defined in ctgclient.h they are:
 * \li #CTG_TRACE_LEVEL0 = Disable all trace
 * \li #CTG_TRACE_LEVEL1 = Exception trace
 * \li #CTG_TRACE_LEVEL2 = Event trace
 * \li #CTG_TRACE_LEVEL3 = Entry/exit trace
 * \li #CTG_TRACE_LEVEL4 = Debug trace
 *
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_NULLPTR
 */
int CICSCALL CTG_getAPITraceLevel(int * traceStatePtr);

/**
 * Set the ECI version 2 API trace level. Takes an int variable
 * representing the desired trace level and sets the API trace level 
 * accordingly. The default trace destination is stderr, but this can be 
 * overridden by the API function CTG_setAPITraceFile(). 
 *
 * The API trace level can also be specified by setting the environment
 * variable CTG_CLIENT_TRACE_LEVEL to the trace level required. Valid
 * values are 0 to 4. The environment variable must be set before the
 * application is started. Use of this function overrides any value
 * obtained from the environment variable.
 *
 * @param[in] traceState an input value, an int variable set to the 
 *       required trace level.
 * \n Valid values are defined in ctgclient.h they are:
 * \li #CTG_TRACE_LEVEL0 = Disable all trace
 * \li #CTG_TRACE_LEVEL1 = Exception trace
 * \li #CTG_TRACE_LEVEL2 = Event trace
 * \li #CTG_TRACE_LEVEL3 = Entry/exit trace
 * \li #CTG_TRACE_LEVEL4 = Debug trace

 * @returns An integer value identifies whether the call was successful 
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_TRACELEVEL
 */
int CICSCALL CTG_setAPITraceLevel(int traceState);

/**
 * Takes a character pointer to a null terminated string representing 
 * the desired trace file destination. If the specified file already 
 * exists, trace data is appended. If the API is unable to open the 
 * specified file for writing, then the trace destination remains 
 * unchanged. Specifying a null pointer causes the trace destination to 
 * revert to the default destination of stderr. 
 *
 * The trace file can also be specified by setting the environment
 * variable CTG_CLIENT_TRACE_FILE to a filename. Any path/filename that 
 * is valid for the operating system is an acceptable value. The 
 * environment variable must be set before the application is started. 
 * Use of this function overrides any value obtained from the 
 * environment variable. 
 *
 * @param[in] traceFileNamePtr char pointer to a null terminated
 * string representing the desired trace destination,
 * path+filename.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_TRACEFILE
 */
int CICSCALL CTG_setAPITraceFile(char * traceFileNamePtr);

/**
 * Sets the maximum length of data that is displayed in ECI version 2
 * API trace. The function takes a size_t parameter which specifies
 * the maximum amount of data to trace.
 *
 * The trace data length can also be specified by setting the
 * environment variable CTG_CLIENT_DATA_LENGTH to a positive numeric
 * value. The environment variable must be set before the application is
 * started. Use of this function overrides any value obtained from
 * the environment variable.
 *
 * \note The IBM service organization may request that this function is 
 *       used to aid problem determination.
 *
 * @param[in] dataLength Maximum length of data to display in trace.
 */
void CICSCALL CTG_setAPITraceDataLength(size_t dataLength);

/**
 * Sets the offset into data that is displayed in ECI version 2 API
 * trace. The function takes a size_t parameter which specifies the
 * offset into the data.
 *
 * The trace data offset can also be specified by setting the
 * environment variable CTG_CLIENT_DATA_OFFSET to a positive numeric
 * value. The environment variable must be set before the application is
 * started. Use of this function overrides any value obtained from
 * the environment variable.
 *
 * \note The IBM service organization may request that this function is 
 *       used to aid problem determination.
 *
 * @param[in] dataOffset Offset into data to begin tracing.
 */
void CICSCALL CTG_setAPITraceDataOffset(size_t dataOffset);

/**
 * Writes CICS TG internal debugging information to the trace destination. The
 * debug information is written irrespective of trace settings.
 * \note The IBM service organization may request that this function is 
 *       used to aid problem determination.
 *
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_dumpState();

/**
 * Gets the string representation of the return code.
 *
 * @param[in] returnCode an integer value for naming.
 * @param[out] rcString a string identifier naming the return code. 
 * @returns a pointer to rcString.
 */
char * CICSCALL CTG_getRcString(int returnCode, char * rcString);

/**
 * Gets the API version. This will only change when the API changes.
 *
 * @param[out] *apiVersPtr a pointer to the api version string. The 
 * returned string (returned in the pointer) is owned by the API, and 
 * should not be free'd by the application. 
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_getAPIVersion(char ** apiVersPtr);

/**
 * Gets the DLL version. This will change when the build level changes.
 *
 * @param[out] *dllVersPtr a pointer to the dll version string. The 
 * returned string (returned in the pointer) is owned by the API, and 
 * should not be free'd by the application. 
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_NULLPTR
 * \li #CTG_ERR_MALLOCFAIL
 */
int CICSCALL CTG_getDLLVersion(char ** dllVersPtr);

/*****************************************************************************/
/**
 * The data for a server as returned from calling CTG_listSystems.
 * CTG_listSystem_t is used to hold the name and description of a server 
 * definition. An array of CTG_listSystem_t is used when CTG_listSystems 
 * is called.
 */
typedef struct
{
   /* Length of SystemName */
   #define CTG_LIST_SYSTEM_LENGTH        8
   /**
   * SystemName identifies the name of the server. When SystemName is 
   * shorter than CTG_LIST_SYSTEM_LENGTH SystemName is padded with nulls to 
   * a length of CTG_LIST_SYSTEM_LENGTH + 1. 
   */
   char SystemName[CTG_LIST_SYSTEM_LENGTH+1];

   /* Length of the Description */
   #define CTG_LIST_DESCRIPTION_LENGTH  60
   /**
   *  Description is the description of the server identified by 
   *  SystemName.  When the description is shorter than
   *  CTG_LIST_DESCRIPTION_LENGTH  characters, it is padded with nulls
   *  to a length of CTG_LIST_DESCRIPTION_LENGTH + 1.
   */
   char Description[CTG_LIST_DESCRIPTION_LENGTH+1];
} CTG_listSystem_t;

/*****************************************************************************/
/**
 *  CTG_listSystems returns a list of all the systems known to the
 *  Gateway, including IPIC and logical CICS servers (only available on
 *  z/OS).  Although the list of systems are known to the Gateway there
 *  is no guarantee that a communications link exists or that any of the
 *  servers is available *  to process requests.
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param Systems Used for input and output, on input the maximum number
 * of systems allowed for in List, on return the actual number of systems
 * found.
 * @param[out] List on return, the reference to an array of
 * CTG_ListSystem_t structures containing the system names and
 * descriptions.
 * @returns An integer value identifies whether the call was successful
 * or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADGWTOKLIST
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_NULLECIPOINTER
 * \li #CTG_ERR_UNKNOWN_REQUEST_TYPE
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 * \li #CTG_ERR_MORE_SYSTEMS
 * \li #CTG_ERR_NO_SYSTEMS
 * \li #CTG_ERR_INVALID_DATA_LENGTH
 * \li #CTG_ERR_SYSTEM_ERROR
 *
*/
int CICSCALL CTG_listSystems(CTG_ConnToken_t gwTok,
                             unsigned short *Systems,
                             CTG_listSystem_t *List);

/*********************************************************************/
/*          CICS TG ECI Version 2 Return Code constants              */
/*********************************************************************/
/**
 * The API call completed successfully.
 */
#define CTG_OK                        0

/** The data length was not valid. */
#define CTG_ERR_INVALID_DATA_LENGTH  -1

/** An internal system error has occurred. */
#define CTG_ERR_SYSTEM_ERROR         -9

/** There are more systems to return than were requested.   */
#define CTG_ERR_MORE_SYSTEMS         -25

/** There are no servers known to the Gateway daemon.       */
#define CTG_ERR_NO_SYSTEMS           -26

/**
 * The Gateway token is NULL.
 */
#define CTG_ERR_NULLGWTOK          -100
/**
 * The Gateway token was not recognized by the Client API. The
 * token is either not valid, or the connection to the Gateway
 * daemon has been lost and the token deleted.
 */
#define CTG_ERR_BADGWTOK           -101
/**
 * The connection to the Gateway daemon has been lost.
 */
#define CTG_ERR_LOSTGWCON          -102
/**
 * One of the parameters is NULL.
 */
#define CTG_ERR_NULLPARM           -103
/**
 * The trace level is not valid.
 */
#define CTG_ERR_TRACELEVEL         -104
/**
 * The port number is not valid. Port numbers must be in the
 * range 1 through 65535.
 */
#define CTG_ERR_BADPORT            -107
/**
 * The Gateway token pointer is NULL.
 */
#define CTG_ERR_NULLGWTOKPTR       -108
/**
 * The pointer is NULL.
 */
#define CTG_ERR_NULLPTR            -109
/**
 * The API was unable to allocate sufficient memory to complete the
 * operation.
 */
#define CTG_ERR_MALLOCFAIL         -111
/**
 * Internal error with the Gateway token list.
 */
#define CTG_ERR_BADGWTOKLIST       -115
/**
 * Failed to establish a connection with the Gateway daemon.
 */
#define CTG_ERR_CONNECTFAILED      -118
/**
 * Deprecated. No longer used.
 * #define CTG_ERR_ICONV           -120
 */
/**
 * Deprecated. No longer used.
 */
#define CTG_ERR_LOCKFAIL           -121
/**
 * Deprecated. No longer used.
 * #define CTG_ERR_COMMSFAIL       -122
 */
/**
 * The Gateway token does not belong to this process.
 */
#define CTG_ERR_PIDMISMATCH        -123
/**
 * The Gateway token does not belong to this thread.
 */
#define CTG_ERR_TIDMISMATCH        -124
/**
 * Unable to create a trace file at the specified location.
 */
#define CTG_ERR_TRACEFILE          -130
/**
 * The hostname did not resolve to a valid IP address.
 */
#define CTG_ERR_BADHOST            -132
/**
 * The hostname is NULL.
 */
#define CTG_ERR_NULLADDRESS        -133
/**
 * The connection timed out.
 */
#define CTG_ERR_CONNECTTIMEOUT     -134
/**
 * The ECI parameter block pointer is NULL.
 */
#define CTG_ERR_NULLECIPOINTER     -137
/**
 * The timeout parameter is not in the range 0 through 3000.
 */
#define CTG_ERR_INVALIDTIMEOUTPARM -138
/**
 * The Gateway token has been closed and no new work can be
 * sent to the Gateway daemon.
 */
#define CTG_ERR_GWTOK_CLOSED -139
/**
 * The request type is not recognized by the Gateway daemon.
 */
#define CTG_ERR_UNKNOWN_REQUEST_TYPE 0xF002
/**
 * The Gateway daemon is closing.
 */
#define CTG_ERR_GATEWAY_CLOSED 0xF004
/**
 * The Gateway daemon refused the request because no worker threads 
 * were available or the Gateway daemon is closing.
 */
#define CTG_ERR_WORK_WAS_REFUSED 0xF005
/**
 * An internal exception occurred in the Gateway daemon.
 */
#define CTG_ERR_GATEWAY_EXCEPTION 0xF006
/**
 * Deprecated. No longer used.
 * #define CTG_ERR_NOT_AUTHORIZED 0xF007
 */
/**
 * The connection failed because the Gateway daemon is at an earlier 
 * product release than the Client application. 
 */
#define CTG_ERR_GATEWAY_BACK_LEVEL 0xF00A
/**
 * The protocol handler for the port that the Client application is 
 * connected to does not accept the request type sent. 
 */
#define CTG_ERR_INVALID_REQUEST_TYPE 0xF00B

/*********************************************************************/
/*            CICS TG ECI Version 2 Trace constants                  */
/*********************************************************************/
/**
 * Trace level 0 disables all tracing.
 */
#define CTG_TRACE_LEVEL0              0
/**
 * Trace level 1 includes exception trace points only.
 */
#define CTG_TRACE_LEVEL1              1
/**
 * Trace level 2 includes event trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL2              2
/**
 * Trace level 3 includes entry/exit trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL3              3
/**
 * Trace level 4 includes debug trace points + lower trace
 * levels.
 */
#define CTG_TRACE_LEVEL4              4


#endif /* ifndef _CTGCLIENT_H */

/* Doxygen index page  */

/*! \mainpage CICS TG ECI Version 2 for C Overview - Supportpac CA0B
 *
 * This documentation is for the CICS Transaction Gateway ECI Version
 * 2 API for C.  This application programming interface (API) provides
 * support for C-language ECI applications to access CICS servers from
 * multiplatform machines remote to the Gateway daemon. These
 * applications must connect to a Gateway daemon over TCP/IP before
 * flowing an ECI request to the target CICS server.
 *
 * This version of the API is supported on AIX only.
 *
 * The API is not supported on z/OS but connectivity from a remote ECI
 * application to CICS TG for z/OS is supported.
 *
 * ECI version 2 C sample programs are provided with SupportPac CA0B,
 * the samples are installed in directory
 * <b> &lt;supportpac_path&gt;/AIX/samples/c/eci_v2 </b>
 *
 * \htmlonly
 * <b>Helpful links</b>
 * <ul>
 * <li><a href="globals_func.html">API Functions</a>
 * <li><a href="structCTG__ECI__PARMS.html">ECI Structure</a>
 * </ul>
 * \endhtmlonly
 *
 * The types and functions that make up this API are defined in header
 * files ctgclient.h and ctgclient_eci.h.
 *
 * When running on AIX, the ECI version 2 API disables signal handling
 * for the SIGPIPE signal.
 *
 * When an application written for ECI version 2 is ready for deployment,
 * the appropriate CICS TG DLL or shared object file must be made
 * available to the remote system. The shared object file for deployment with
 * the application is:
 * \htmlonly
 * <table>
 * <tr><td> AIX                       </td>
 *     <td> &lt;supportpac_path&gt;/AIX/lib64/libctgclient.a  </td></tr>
 * </table>
 * \endhtmlonly
 *
 * \note The Data Structures and Class Hierarchy sections contain the same
 * information. Because the ECI Version 2 API is for the C-language, the Class
 * Hierarchy section can be ignored.
 */

