/*
   MODULE NAME        CTGCLIENT_ECI.H

   DESCRIPTIVE NAME   CICS Transaction Gateway ECI Version 2
                      Application Programming Interface, ECI Parts
                      SupportPac CA0B

   Licensed Materials - Property of IBM
   
   Supportpac CA0B
      
   (c) Copyright IBM Corp. 2011, 2012 All Rights Reserved.
   
   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with
   IBM Corp.
   
   NOTES :-

   To ensure the correct sections of this header are included, the
   following constant must be defined at compile time:

     CICS_AIX  -  if building an AIX application
*/
/**
* @file
* Defines functions and typedefs for sending External Call Interface
* (ECI) requests to a Gateway daemon.
*/

#ifndef _CTG_ECI_H
#define _CTG_ECI_H

#include "ctgclient.h"

#if defined (CICS_W32) || defined(CICS_WNT)
  #define CICSCALL __cdecl
#else
  #define CICSCALL
#endif

#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

/*****************************************************************************/
/** The Version 2 External Call Interface (ECI) parameter block.
 *  The CTG_ECI_PARMS structure is the parameter block used when
 *  communicating with a CICS server. The parameter block fields are
 *  used for input and output. To communicate with the CICS server
 *  through the Gateway daemon use the CTG_ECI_Execute() function.
 *
 *  Set the ECI parameter block to nulls before setting the input
 *  parameter fields.
 *
 *  Guidance on how to use the ECI to manage logical units of work can
 *  be found in the CICS Transaction Gateway Information Center:
 *  Programming Guide, search for "Managing logical units of work".
 *
 *  The ECI parameter block is transmitted to the Gateway daemon without
 *  any code page conversion. For more information on managing code
 *  pages refer to the Information Center: Programming Guide, search
 *  for title "Data conversion".
 *
 *  \note Text strings are NOT null-terminated.
 *
 */
typedef struct
{
   /* Values used with eci_call_type */
   #define ECI_SYNC                     1
   #define ECI_ASYNC                    2
   #define ECI_GET_REPLY                3
   #define ECI_GET_REPLY_WAIT           4
   #define ECI_GET_SPECIFIC_REPLY       5
   #define ECI_GET_SPECIFIC_REPLY_WAIT  6
   /** \brief Identifies the type of ECI flow.
    *  A required input parameter. The call type identifies the type of
    *  the ECI flow. The valid values are :
    *
    *  \li ECI_SYNC<br>
    *  The ECI_SYNC call type provides a synchronous program link call to
    *  start, continue, or complete a logical unit of work. The calling
    *  application does not get control back until the called CICS
    *  program has run to completion.
    *  \li ECI_ASYNC<br>
    *  The ECI_ASYNC call type provides and asynchronous program link
    *  call to start, continue, or complete a logical unit of work.
    *  Control is returned to the application once the ECI request has
    *  been sent to the Gateway daemon.
    *  \li ECI_GET_REPLY<br>
    *  The ECI_GET_REPLY call type provides a reply solicitation call that
    *  returns one of the available replies. If there is no reply
    *  available, then ECI_ERR_NO_REPLY is returned.
    *  \li ECI_GET_REPLY_WAIT<br>
    *  The ECI_GET_REPLY_WAIT call type provides a reply solicitation call
    *  that returns one of the available replies. If there is no reply
    *  available, then the call waits until a reply is available.
    *  \li ECI_GET_SPECIFIC_REPLY<br>
    *  The ECI_GET_SPECIFIC_REPLY call type provides a reply solicitation
    *  call that returns the reply for the asynchronous request identified
    *  by eci_message_qualifier. If there is no reply available, then
    *  ECI_ERR_NO_REPLY is returned.
    *  \li ECI_GET_SPECIFIC_REPLY_WAIT<br>
    *  The ECI_GET_SPECIFIC_REPLY_WAIT call type provides a reply
    *  solicitation call that returns the reply for the asynchronous
    *  request identified by eci_message_qualifier. If there is no reply
    *  available, the call waits until the reply is available.
    */
   signed short   eci_call_type;

   #define ECI_PROGRAM_NAME_LENGTH 8
   /** \brief  CICS program to run.
    *  
    *  An input parameter, the CICS program name is required except when
    *  eci_extend_mode is ECI_COMMIT or ECI_BACKOUT.
    *
    *  An 8 character field containing the name of the program to be
    *  called. Pad unused characters with spaces or nulls. To avoid code
    *  page conversion issues it is recommended to use uppercase
    *  characters A to Z, and the numeric characters 0 to 9.  For more
    *  information on managing code pages refer to the Information
    *  Center: Programming Guide, search for title "Data conversion". */
   char  eci_program_name[ECI_PROGRAM_NAME_LENGTH];

   #define ECI_USERID_LENGTH       16
   /** \brief  Userid for security checking.
    *  
    *  The eci_userid has a maximum length of 16 characters. */
   char  eci_userid[ECI_USERID_LENGTH];

   #define ECI_PASSWORD_LENGTH     16
   /** \brief  Password for security checking.
    *  
    *  The eci_password has a maximum length of 16 characters. */
   char  eci_password[ECI_PASSWORD_LENGTH];

   #define ECI_TRANSID_LENGTH      4
   /** \brief CICS transaction identifier.
    *
    *  A 4 character field optionally containing the ID of a CICS
    *  transaction. Pad unused characters with spaces or nulls. The
    *  parameter is ignored if eci_tpn is used (set to any value other
    *  than nulls). The use of this parameter depends on the client from
    *  which the request is sent. The value of eci_transid is converted
    *  from ASCII to EBCDIC, with no uppercase translation, and stored
    *  in EIBTRNID for the duration of the LINK to the program specified
    *  in the eci_program_name.
    *
    *  The called program runs under the mirror transaction CPMI, but is
    *  linked to under the eci_transid transaction name. This name is
    *  available to the called program for querying the transaction ID.
    *  Some servers use the transaction ID to determine security and
    *  performance attributes for the called program. In those servers,
    *  use this parameter to control the processing of your called
    *  programs. */
   char  eci_transid[ECI_TRANSID_LENGTH];

   #define ECI_ABEND_CODE_LENGTH   4
   /** \brief CICS abend code.
   *
   *   An output parameter, a 4 character field in which a CICS abend
   *   code is returned. If the transaction that executes the called
   *   program does not abend spaces are returned. */
   char  eci_abend_code[ECI_ABEND_CODE_LENGTH];

   /** \brief The data passed to and returned from the CICS program.
    *
    *  An optional input parameter, a pointer to the data to be passed
    *  as the COMMAREA of the CICS program identified by
    *  eci_program_name. If no COMMAREA is required, supply a null
    *  pointer and set the eci_commarea_length to zero. The COMMAREA is
    *  also an output field, it can be used by the called program to
    *  return information to the application. A COMMAREA can only be
    *  returned from the program, if one was sent */
   void *      eci_commarea;

   /** \brief The length of the eci_commarea.
    *
    *  The length of the COMMAREA in bytes. This is used by the
    *  interface as the maximum length of the eci_commarea. The COMMAREA
    *  lengths for sending and receiving data should be defined by
    *  commarea_outbound_length and commarea_inbound_length.
    *
    *  Application developers are advised to use a maximum size of 
    *  32500 bytes, as this is guaranteed to be flowed successfully
    *  across all protocols. COMMAREA sizes greater than this may
    *  generate an ECI_ERR_INVALID_DATA_LENGTH return code. */

   signed short   eci_commarea_length;

   /** \brief Timeout in seconds.
   *
   *   The time in seconds to wait for a response from the CICS
   *   server. A value of zero means that no limit is set.
   *
   *   If timeout occurs, the conversation is abended. */
   signed short   eci_timeout;

   /* Values used with eci_extend_mode */
   #define ECI_NO_EXTEND        0
   #define ECI_EXTENDED         1
   #define ECI_COMMIT           2
   #define ECI_BACKOUT          4
   /** \brief Extend mode defines the behaviour of the ECI flow.
   *
   *  A required input parameter that defines how a logical unit
   *  of work is processed.
   *
   *  The values for this field, shown by their symbolic names,
   *  are as follows:
   *  <UL>
   *  <li>     ECI_NO_EXTEND
   *   <OL><li> If the input eci_luw_token field is zero it is
   *           the only call for a logical unit of work.
   *       <li> If the input eci_luw_token field is not zero it
   *            is the last call for the specified logical unit of
   *            work.
   *  </OL>
   *  In each case, changes to recoverable resources are committed
   *  by a CICS end-of-task sync point, and the logical unit of
   *  work ends.
   *
   *  <li>      ECI_EXTENDED
   *   <OL><li>  If the input eci_luw_token field is zero, it is
   *             the first call for a logical unit of work that is
   *             to be continued.
   *       <li>  If the input eci_luw_token field is not zero, this
   *             call is intended to continue the specified logical
   *             unit of work.
   *  </OL>
   *  In each case the logical unit of work continues after the
   *  called program completes successfully, and changes to
   *  recoverable resources remain uncommitted.
   *
   *   <li>      ECI_COMMIT
   *  \n Terminate the current logical unit of work, identified by
   *  the input eci_luw_token field, and commit all changes made to
   *  recoverable resources.
   *
   *   <li>      ECI_BACKOUT
   *  \n Terminate the logical unit of work identified by the input
   *  eci_luw_token field, and back out all changes made to
   *  recoverable resources.
   *  </UL>
   *
   *  For guidance on how to use the ECI to manage logical units of
   *  work refer to the CICS Transaction Gateway Information
   *  Center: Programming Guide, search for "Managing logical units
   *  of work".
   */
   signed short   eci_extend_mode;

   /* Used with eci_luw_token to request a new logical unit of work */
   #define  ECI_LUW_NEW 0
   /**  \brief This token identifies the logical unit of work to
    *   which the ECI flow is associated.
    *
    * An integer field is used for identifying the logical unit of
    * work to which a call belongs. The eci_luw_token must be set
    * to ECI_LUW_NEW or 0 at the start of a logical unit of work,
    * regardless of whether the logical unit of work is going to be 
    * extended. If the logical unit of work is to be extended, the 
    * ECI updates eci_luw_token with a valid value on the first 
    * call of the logical unit of work, the returned value must be 
    * used as the input value for all subsequent calls that are 
    * part of the same logical unit of work. 
    *
    * If the return code is not ECI_NO_ERROR, and the call was
    * continuing or ending an existing logical unit of work, this
    * field is used as output to report the condition of the
    * logical unit of work. If it is set to 0, the logical unit of
    * work has ended and its updates have been backed out. If it is 
    * nonzero, it is the same as the input value, the logical unit 
    * of work is continuing, and its updates are still pending. The 
    * eci_luw_token field must be set to 0 when eci_extend_mode is 
    * ECI_NO_EXTEND */ 
   signed int     eci_luw_token;

   /* Values used with eci_version */
   #define ECI_VERSION_2        4
   #define ECI_VERSION_S1     100
   /** \brief ECI version identifier.
    *
    *  A required input parameter that identifies the version of
    *  the ECI for which the application is coded. This field must
    *  be set to ECI_VERSION_2 or ECI_VERSION_S1. ECI_VERSION_2 is
    *  provided for compatibility with existing applications that use
    *  only the ECI_SYNC call type. ECI_VERSION_S1 must be used for
    *  asynchronous call support.
    */
   signed short   eci_version;

   #define ECI_SYSTEM_NAME_LENGTH  8
   /** \brief Server name. An optional input parameter.
   *
   *   An 8 character field that specifies the name of the server
   *   to which the ECI request is to be directed. If supplied, it
   *   must be one of the server names returned by
   *   CTG_listSystems, with the exception that, if connected to a
   *   z/OS gateway, it is possible to pass the APPLID of a CICS
   *   region which the gateway then communicates with using EXCI.
   *   Pad unused characters with spaces or nulls. If the server
   *   name is supplied when eci_luw_token is not zero, the server
   *   name is ignored, as all requests within a logical unit of
   *   work must be sent to the same server.
   *
   *   If the field is set to nulls, the default server defined
   *   in the Gateway daemon is used. */
   char     eci_system_name[ECI_SYSTEM_NAME_LENGTH];

   #define ECI_TPN_LENGTH          4
   /** \brief A transaction identifier for the mirror transaction.
    *
    *  Optional input parameter. A 4-character field that specifies
    *  the transaction ID that is used by the server to process the
    *  ECI request. This transaction must be defined in the server
    *  as a CICS mirror transaction. If the field is not set, the
    *  default mirror transaction CPMI is used.
    *
    *  If the ECI request is extended (see the description of
    *  eci_extend_mode), this parameter has a meaning only for the
    *  first request.
    *
    *  If this field is set, the contents of eci_transid are
    *  ignored. */
   char     eci_tpn[ECI_TPN_LENGTH];

   /** \brief Maximum data length to send to the Gateway daemon.
    *  
    *  If commarea_outbound_length is set to 0 or is greater than
    *  eci_commarea_length, the value of eci_commarea_length is
    *  used instead of commarea_outbound_length. */
   signed short commarea_outbound_length;

   /** \brief Maximum data length to receive from the Gateway daemon.
   *  
   *  If the commarea_inbound_length is set to 0 or is greater than
   *  eci_commarea_length, the value of eci_commarea_length is
   *  used instead of commarea_inbound_length.
   */
   signed short commarea_inbound_length;

   /**
    * \brief Identifier for an asynchronous request reply. 
    *  
    * For ECI_ASYNC calls, this field is an output parameter 
    * containing the identifier that can be used to get the 
    * response to this specific request. 
    *  
    * For ECI_GET_SPECIFIC_REPLY and ECI_GET_SPECIFIC_REPLY_WAIT 
    * calls, this field is an input parameter specifying the 
    * identifier of the asynchronous response to be returned. 
    *  
    * For other call types, this field is not used. 
    */
   signed int eci_message_qualifier;

   /** \internal Reserved field */
   void * reserved1;
} CTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  A reference for an ECI Parameter block, CTG_ECI_Execute .
 */
typedef CTG_ECI_PARMS *PCTG_ECI_PARMS;


/*****************************************************************************/
/**
 *  Sends an ECI request to the Gateway.
 *  Equivalent to the flow() method of a com.ibm.ctg.client.JavaGateway object.
 *
 * @param[in] gwTok The reference to the open Gateway connection.
 * @param EciParms The reference to the CTG_ECI_PARMS structure, the
 * fields of the structure are used for input and output.
 *
 * @returns An integer value identifies whether the call was successful or not.
 * \n Possible values include
 * \li #CTG_OK
 * \li #CTG_ERR_MALLOCFAIL
 * \li #CTG_ERR_BADGWTOKLIST
 * \li #CTG_ERR_NULLGWTOKPTR
 * \li #CTG_ERR_NULLPARM
 * \li #CTG_ERR_LOSTGWCON
 * \li #CTG_ERR_NULLECIPOINTER
 * \li #CTG_ERR_UNKNOWN_REQUEST_TYPE
 * \li #CTG_ERR_GATEWAY_CLOSED
 * \li #CTG_ERR_WORK_WAS_REFUSED
 * \li #CTG_ERR_GATEWAY_EXCEPTION
 * \li #CTG_ERR_INVALID_REQUEST_TYPE
 *
 * Other possible values are all of the documented ECI return codes.
 *
*/
int CICSCALL CTG_ECI_Execute(CTG_ConnToken_t gwTok,
                             CTG_ECI_PARMS *EciParms);



/*********************************************************************/
/*                CICS TG ECI Return Code constants                  */
/*********************************************************************/
/**
 * No error.
 */
#define ECI_NO_ERROR                  0

/**
 * Bad data length. This might be because the protocol used to 
 * communicate with CICS is unable to send or receive the specified 
 * COMMAREA size. 
 */
#define ECI_ERR_INVALID_DATA_LENGTH  -1

/**
 * The eci_extend_mode value is not valid.
 */
#define ECI_ERR_INVALID_EXTEND_MODE  -2

/**
 * Communication with the target CICS server could not be established.
 */
#define ECI_ERR_NO_CICS              -3

/**
 * Communication with the target CICS server was lost.
 */
#define ECI_ERR_CICS_DIED            -4

/**
 * The eci_timeout interval expired before the request was sent 
 * to CICS, or the timeout value is negative.
 */
#define ECI_ERR_REQUEST_TIMEOUT      -5

/**
 * The eci_timeout interval expired while waiting for a 
 * response. 
 */
#define ECI_ERR_RESPONSE_TIMEOUT     -6

/**
 * The CICS transaction abended. The abend code is available in the
 * eci_abend_code field.
 */
#define ECI_ERR_TRANSACTION_ABEND    -7

/**
 * The value in eci_luw_token is incorrect. There is no active unit
 * of work associated with the specified token.
 */
#define ECI_ERR_LUW_TOKEN            -8

/**
 * An internal system error has occurred. If the problem persists,
 * collect application and Gateway daemon trace and contact your IBM
 * service representative.
 */
#define ECI_ERR_SYSTEM_ERROR         -9

/**
 * The eci_call_type value is not valid.
 */
#define ECI_ERR_INVALID_CALL_TYPE    -14

/**
 * An attempt was made to continue an existing logical unit of
 * work, but there was an outstanding asynchronous call for the 
 * same logical unit of work. 
 */
#define ECI_ERR_ALREADY_ACTIVE       -15

/**
 * There are insufficient resources to communicate with the target
 * CICS server; details might be available in the CICS TG error log
 * files.
 */
#define ECI_ERR_RESOURCE_SHORTAGE    -16

/**
 * There are insufficient communication resources to communicate with
 * the target CICS server.
 */
#define ECI_ERR_NO_SESSIONS          -17

/**
 * The combination of eci_commarea and eci_commarea_length values
 * is not valid. If eci_commarea is NULL, eci_commarea_length must be
 * zero. If eci_commarea is not NULL, eci_commarea_length must not be
 * zero.
 */
#define ECI_ERR_INVALID_DATA_AREA    -19

/**
 * The eci_version value is not valid.
 */
#define ECI_ERR_INVALID_VERSION      -21

/**
 * The target CICS server is unknown to the Gateway daemon.
 */
#define ECI_ERR_UNKNOWN_SERVER       -22

/**
 * The combination of user ID and password is not valid.
 */
#define ECI_ERR_SECURITY_ERROR       -27

/**
 * The limit of the maximum number of connected CICS servers has been 
 * reached. 
 */
#define ECI_ERR_MAX_SYSTEMS          -28

/**
 * There are insufficient communication resources to complete the
 * request.
 */
#define ECI_ERR_MAX_SESSIONS         -29

/**
 * An attempt was made to commit a logical unit of work, but the
 * CICS server was unable to commit the changes and backed them 
 * out. 
 */
#define ECI_ERR_ROLLEDBACK           -30

/** For ECI_GET_REPLY calls this return code indicates
 * that there are no replies outstanding. For ECI_GET_SPECIFIC_REPLY 
 * calls this return code indicates that there is no reply for 
 * the specified message qualifier. 
 */
#define ECI_ERR_NO_REPLY             -32

/** For ECI_GET_SPECIFIC_REPLY_WAIT calls this return code indicates that
 * the message qualifier is not in use.
 */
#define ECI_ERR_INVALID_MSG_QUAL     -33

#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#endif /* _CTG_ECI_H */

