/*
   File name(s)  : ctgutils.h

   Product       : CICS Transaction Gateway Supportpac CA0B

   Description   : This include file provides thread, event and mutex types
                   and functions for each platform which are required for
                   use in the some samples.  See each function for a
                   description of its use.

                   Note there is only minimal error checking and no error
                   reporting in these functions.

    Licensed Materials - Property of IBM

    Supportpac CA0B

    (c) Copyright IBM Corp. 2012

    US Government Users Restricted Rights - Use, duplication or
    disclosure restricted by GSA ADP Schedule Contract with
    IBM Corp.

    Status: Version 1 Release 1

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */

#ifndef _CTGUTILS_H
#define _CTGUTILS_H


#if !defined(CICS_W32) && !defined(CICS_AIX) && !defined(CICS_SOL) && !defined(CICS_HPUX) && !defined(CICS_LNX)
#error Must compile with a compiler define of CICS_xxx (W32, AIX, SOL, HPUX or LNX)
#endif

/* Include files */

#include <stdlib.h>

#if defined(CICS_W32)
   #include <windows.h>
   #include <process.h>
   #include <string.h>
   #include <sys/types.h>
   #include <sys/timeb.h>

#else
   #include <pthread.h>
   #include <sys/time.h>
   #include <errno.h>
#endif

/* Constants */
#define CTGTRUE   1
#define CTGFALSE  0

/* Data types */
typedef int                CtgBool;
typedef unsigned long      CtgUINT32;

#if   defined(CICS_W32)
   typedef CtgUINT32       CtgThreadId;
   typedef HANDLE          CtgEvent;
   typedef HANDLE          CtgMutex;
   typedef void (__cdecl *CtgFunction)(void*);

#else
   typedef pthread_t       CtgThreadId;
   typedef struct {
       pthread_cond_t  cond;
       pthread_mutex_t mutex;
       CtgBool         flag;
       struct timeval  postTime;
   } CtgEventStruct;
   typedef CtgEventStruct* CtgEvent;
   typedef pthread_mutex_t CtgMutex;
   typedef void (*CtgFunction)(void*);
#endif


/* Function prototypes */

/* Utility functions */
void waitInMillisecs(int millisecs);
unsigned long curTimeInMilliSecs(void);

/* Threading functions */
CtgThreadId queryMyThreadId(void);
CtgBool startThread(CtgFunction threadFunc, void* param);
void endMyThread(void);

/* Event functions */
CtgBool createEvent(CtgEvent* myEvent);
void deleteEvent(CtgEvent* myEvent);
CtgBool waitOnEvent(CtgEvent* myEvent);
CtgBool resetEvent(CtgEvent* myEvent);
CtgBool flagEvent(CtgEvent* myEvent);

/* Mutex functions */
CtgBool createMutex(CtgMutex* myMutex);
void deleteMutex(CtgMutex* myMutex);
CtgBool acquireMutex(CtgMutex* myMutex);
CtgBool releaseMutex(CtgMutex* myMutex);


/*
   Function: startThread

   Purpose:  Creates a thread which starts executing in 'threadFunc' with
             parameter 'param'.
*/
CtgBool startThread(CtgFunction threadFunc, void* param) {
   CtgThreadId threadId;
   CtgBool     retCode = CTGFALSE;

#if defined(CICS_W32)
   threadId = _beginthread(threadFunc, 16384, param);
   if (threadId != -1) {
      retCode = CTGTRUE;
   }

#else
   pthread_attr_t pattr;

   if (pthread_attr_init(&pattr) != 0) {
	   return retCode;
   }

#if defined(CICS_AIX) || defined(CICS_HPUX)
   /* using 112K stack size */
   if (pthread_attr_setstacksize(&pattr, 114688) != 0) {
      return retCode;
   }
#endif

   if (pthread_create(&threadId, &pattr, (void* (*)(void*))threadFunc, param) == 0) {
      retCode = CTGTRUE;
   }
#endif

   return retCode;
}

/*
   Function: endMyThread

   Purpose:  Terminates the currently executing thread.
*/
void endMyThread() {

#if defined(CICS_W32)
   _endthread();
#else
   pthread_exit(NULL);
#endif
}

/*
   Function: queryMyThreadId

   Purpose:  Returns the thread identifier of the currently executing thread.
*/
CtgThreadId queryMyThreadId() {
   CtgThreadId threadId;

#if defined(CICS_W32)
   threadId = GetCurrentThreadId();
#else
   threadId = pthread_self();
#endif

   return threadId;
}

/*
   Function: createEvent

   Purpose:  Creates an event that threads can wait on and flag, to facilitate
             inter-thread communication. If successful, the event is returned
             in myEvent.
*/
CtgBool createEvent(CtgEvent* myEvent) {
   CtgBool retCode = CTGFALSE;

#if defined(CICS_W32)
   *myEvent = CreateEvent(NULL, CTGTRUE, CTGFALSE, NULL);
   if (*myEvent != NULL) {
      retCode = CTGTRUE;
   }
#else
   CtgEvent event;
   event = (CtgEvent)malloc(sizeof(CtgEventStruct));
   if ((pthread_cond_init(&event->cond, NULL) == 0) &&
         (pthread_mutex_init(&event->mutex, NULL) == 0)) {
      event->flag = CTGFALSE;
      event->postTime.tv_sec = 0;
      event->postTime.tv_usec = 0;
      retCode = CTGTRUE;
      *myEvent = event;
   }
#endif
   return retCode;
}

/*
   Function: deleteEvent

   Purpose:  Deletes an event.
*/
void deleteEvent(CtgEvent* myEvent) {
   if (myEvent != NULL) {
#if defined(CICS_W32)
      CloseHandle(*myEvent);
#else
      CtgEvent event = *myEvent;
      *myEvent = NULL;
      pthread_cond_destroy(&event->cond);
      pthread_mutex_destroy(&event->mutex);
      free(event);
#endif
   }
}

/*
   Function: waitOnEvent

   Purpose:  Suspends the current thread until the specified event is flagged.
*/
CtgBool waitOnEvent(CtgEvent* myEvent) {
   CtgBool retCode = CTGFALSE;
   CtgUINT32 rc;

#if defined(CICS_W32)
   rc = WaitForSingleObject(*myEvent, INFINITE);
   if (rc != WAIT_FAILED) {
      retCode = CTGTRUE;
   }
#else
   CtgEvent event = *myEvent;
   struct timeval waitStart;

   if (pthread_mutex_lock(&event->mutex) == 0) {
      retCode = CTGTRUE;
      if (event->flag == CTGFALSE) {
         if (gettimeofday(&waitStart, NULL) == 0) {
            do {
               if (pthread_cond_wait(&event->cond, &event->mutex) != 0) {
                  retCode = CTGFALSE;
                  break;
               }
            } while ((waitStart.tv_sec > event->postTime.tv_sec) ||
                     ((waitStart.tv_sec == event->postTime.tv_sec) &&
                      (waitStart.tv_usec > event->postTime.tv_usec)));
         } else {
            retCode = CTGFALSE;
         }
      }
      pthread_mutex_unlock(&event->mutex);
   }
#endif
   return retCode;
}

/*
   Function: flagEvent

   Purpose:  Signals that the specified event has occurred, waking any threads
             currently waiting on the event.
*/
CtgBool flagEvent(CtgEvent* myEvent) {
   CtgBool retCode = CTGFALSE;
   CtgUINT32 rc;

#if defined(CICS_W32)
   rc = SetEvent(*myEvent);
   if (rc != 0) {
      retCode = CTGTRUE;
   }
#else
  CtgEvent event = *myEvent;
   if (pthread_mutex_lock(&event->mutex) == 0 &&
       gettimeofday(&event->postTime, NULL) == 0) {
      event->flag = CTGTRUE;
      if (pthread_cond_broadcast(&event->cond) == 0) {
         retCode = CTGTRUE;
      }
      pthread_mutex_unlock(&event->mutex);
   }
#endif
   return retCode;
}

/*
   Function: resetEvent

   Purpose:  Resets the specified event to the non-flagged state, so that
             future calls to waitOnEvent will wait.
*/
CtgBool resetEvent(CtgEvent* myEvent) {
   CtgBool retCode = CTGFALSE;
   CtgUINT32 rc;

#if defined(CICS_W32)
   rc = ResetEvent(*myEvent);
   if (rc != 0) {
      retCode = CTGTRUE;
   }
#else
   CtgEvent event = *myEvent;
   if (pthread_mutex_lock(&event->mutex) == 0) {
      event->flag = CTGFALSE;
      retCode = CTGTRUE;
      pthread_mutex_unlock(&event->mutex);
   }
#endif
   return retCode;
}

/*
   Function: createMutex

   Purpose:  Creates a mutex object that can be used to serialize access to
             shared resources. If successful, the mutex is returned in myMutex.
*/
CtgBool createMutex(CtgMutex* myMutex) {
   CtgBool retCode = CTGFALSE;

#if defined(CICS_W32)
   *myMutex = CreateMutex(NULL, CTGFALSE, NULL);
   if (*myMutex != NULL) {
      retCode = CTGTRUE;
   }
#else
   if (pthread_mutex_init(myMutex, NULL) == 0) {
      retCode = CTGTRUE;
   }
#endif
   return retCode;
}

/*
   Function: deleteMutex

   Purpose:  Deletes a mutex object.
*/
void deleteMutex(CtgMutex* myMutex) {
#if defined(CICS_W32)
   CloseHandle(*myMutex);
#else
   pthread_mutex_destroy(myMutex);
#endif
}

/*
   Function: acquireMutex

   Purpose:  Acquires the specified mutex.
*/
CtgBool acquireMutex(CtgMutex* myMutex) {
   CtgBool retCode = CTGFALSE;
   CtgUINT32 rc;

#if defined(CICS_W32)
   rc = WaitForSingleObject(*myMutex, INFINITE);
   if (rc != WAIT_FAILED) {
      retCode = CTGTRUE;
   }
#else
   if (pthread_mutex_lock(myMutex) == 0) {
      retCode = CTGTRUE;
   }
#endif
   return retCode;
}

/*
   Function: releaseMutex

   Purpose:  Releases the specified mutex that was previously acquired by the
             current thread.
*/
CtgBool releaseMutex(CtgMutex* myMutex) {
   CtgBool retCode = CTGFALSE;
   CtgUINT32 rc;

#if defined(CICS_W32)
   rc = ReleaseMutex(*myMutex);
   if (rc != 0) {
      retCode = CTGTRUE;
   }
#else
   if (pthread_mutex_unlock(myMutex) == 0) {
      retCode = CTGTRUE;
   }
#endif
   return retCode;
}

/*
   Function: waitInMillisecs

   Purpose:  Suspends execution of the current thread for the specified number
             of milliseconds.
*/
void waitInMillisecs(int millisecs) {
#if defined(CICS_W32)
   Sleep(millisecs);
#else
   struct timespec abstime;
   pthread_cond_t cond;
   pthread_mutex_t mutex;
   CtgUINT32 rc;
   struct timeval currentTime;
   signed int total_msec;

   if (gettimeofday(&currentTime, NULL) == 0 &&
       pthread_cond_init(&cond, NULL) == 0 &&
       pthread_mutex_init(&mutex, NULL) == 0 &&
       pthread_mutex_lock(&mutex) == 0) {
      total_msec = (currentTime.tv_usec / 1000) + millisecs;
      abstime.tv_sec = currentTime.tv_sec + (total_msec / 1000);
      abstime.tv_nsec = (total_msec % 1000) * 1000000;
      rc = 0;
      while ((rc = pthread_cond_timedwait(&cond, &mutex, &abstime)) != ETIMEDOUT);
      pthread_mutex_unlock(&mutex);
   }
#endif
}

/*
   Function: curTimeInMillisecs

   Purpose:  Returns the current system time in milliseconds.
*/
unsigned long curTimeInMilliSecs() {
#if defined(CICS_W32)
   struct _timeb timeb;
   _ftime(&timeb);
   return ((timeb.time * 1000) + timeb.millitm);
#else
   struct timeval currentTime;
   gettimeofday(&currentTime, NULL);
   return ((currentTime.tv_sec * 1000) + (currentTime.tv_usec / 1000));
#endif
}


#endif
